unit PPKTypes;

interface

uses TypeUnit;



const
   cVersion     = 02; {2nd byte of pkd game, shifted left 2, max ver# is 63 (1-63) }
   dllName      = 'PPKDLV02'; {**sync}
   BinFN        = 'ppkdlv02.dat'; {**sync}
   cMarked      = 1;  {marked/del/mkdDel low order 2 bits of 2nd byte of ppkd game }
   cDeleted     = 2;
   cMarkDel     = 3;

   KnownTags    = 18; {number of PGN tags recognized; see table at bottom}

   cLIMax       = 16382; {Max longints in LIAry}
   MaxMemRecs   = 16000; {PPKSortU, SortFile. Used also in time ests in callbacks}

   BinFS        = 112800; {size of ppkdlv02.dat}

   NotaMax      = 150;  {NA1,NA2}
   MoveMax      = 1114; {MA}
   SfxMax       = 146;  {SA}
   NameMax      = 872;  {NA, NF}
   WordMax      = 584;  {WA, WO}
   PCharStrLen  = 20000; {std str length for all DLL calls}
   ff255        = #255; {start packed game}
   fe254        = #254; {start packed pgn tag data}
   fd253        = #253; {start, packed notation}
   fc252        = #252; {modifier}
   fb251        = #251; {reserved; unused}
   nilTL        = chr($20); {dummy; impossible Tbl/Len combo}
   ECOSLen      = 246*3;


   {--Flag constants (often combined in a single integer) -----------------}
   REO_Compress          = $0000; {FReo:FileReorg... ALL marked Del games subject}
   REO_Mark              = $0100; {FReo: conjuction w/ LIAry}
   REO_Clear             = $0200; {FReo: conjuntion w/ LIAry}
   REO_Reset             = $0300;

   MRK_ReturnVer         = $0000; {MDM:MarkDelMaint, returns creating DLL Ver of game}
   MRK_ReturnMrkDel      = $0040; {MDM:Returns both states as Booleans}
   MRK_SetMrkDel         = $0080; {MDM:Set Mark & Del individually via Booleans}
   MRK_Reset             = $00c0;

   CHK_ErrorOn           = $0000; {notation search: errors on improper check}
   CHK_ErrorOff          = $0020; {notation search: no checking; faster; no interrupts}

   FMT_Closed            = $0000; {notation: 1.e4  (unpacking only) }
   FMT_Verbose           = $0010; {notation: 1. e4 (ditto) }
                                  {XX  YY}
   FIO_CreateOnly        = $0000; {00  00}   {XX 00:notExi; 01:Exist    ; 10:unk}
   FIO_OverwriteExisting = $0005; {01  01}
   FIO_AppendExisting    = $0006; {01  10}   {YY 00:create; 01:overwrite; 10:append}
   FIO_CreateOrOverwrite = $0009; {10  01}
   FIO_CreateOrAppend    = $000a; {10  10}
   FIO_Set               = [0,5,6,9,10]; {keep in sync!}
   {--Flag constants (end) ---------------------------------------------}


   RestrictSet = [#0,TAB,CAR,LNF,#26,fb251..ff255];
   XXXSet      = RestrictSet-[TAB];
   ASCIISet    = [' '..#127];
   T246Set     = [#0..#255]-RestrictSet;
   T150Set     = T246Set-ASCIISet;
   T146Set     = T150Set-[#1..#4];
   T146PASet   = T146Set+ASCIISet; {T146 Plus ASCII}

   {--the following Sets may not be necessary for unpacking--}
   LowCaseSet  = ['a'..'z'];
   UpCaseSet   = ['A'..'Z'];
   NumericSet  = ['0'..'9'];
   AlphaSet    = LowCaseSet + UpCaseSet;
   AlpNumSet   = AlphaSet + NumericSet;


   CCPStr : PChar =  {NDX=StrPos div 4 to become 6-bit coded w/ hi (7) for () code}
      ' ARG ARM AUT BEL BIH BLR BRA BUL CAN CHI CHN COL CRO CUB CZE'+  {15}
         ' DEN ENG ESP EST FIN FRA GEO GER GRE HUN INA IND IRL ISL ISR'+  {30}
         ' ITA KAZ LAT LTU MDA MEX NED NOR PAK PER PHI POL POR ROM RUS'+  {45}
         ' SCO SLO SUI SVK SWE TKM TUR UKR USA UZB VIE YUG RSA*AUS*URS'+  {60}
         ' AZE ISD URU'; {filled up}                                   {63 max}

   ECOS1 : PChar =
   'B22B90B92B31E81B30B83B93B99B85B97C82B96B66A39B67C34B03B51B46'+
   'E87A17A06E97C36A26C39B95C63E59D38B21A27D60B07D41A90E39C48C89'+
   'A05C19B80C50B26E32A48B81C45A30D36D02B50A49A07B87C14A13A12E15'+
   'D35A37B01E12C41C08E92B53B06E60C69B52B17B14B12A81A36A15A10A00'+
   'B13B05';
   ECOS2 : PChar =
   'B00D05A45A40A29A21A14E94D85C30D03C88C11C06C05B57B33B09B08B02'+
   'A46A34A04A01E91E62E61E11D94A47D45D43D37C26D10D00C55A57C18C40'+
   'C15C10B24B78B23B10C65A80A43A32A03E90B18E70E68E67A24B40E41E18'+
   'E14B72D53C49D34D31D17D15C80C77B15C02C00B88B70B45B42E86C37A96'+
   'B25A88';
   ECOS3 : PChar =
   'A87A56C29A41A25A18A11A08A02E76D48E69E63E58E43E38D01E19E73E05'+
   'E00D88D56D52D47D44D30D13D12C97C20C87C84C78C68C67C53C52C51C47'+
   'C43C17D28C01B79B76B58B56B36B35B34D86B27B20B16B04A85B77A68D04'+
   'A61A59D09A55A54A42A33A28A20C42C44E04C33B65B47C62D27C25E42D46'+
   'A84C56'; {246}


type
   TCallBack3    = procedure(stage:integer;iteration,expected:longint);
   {need not be accessable by calling proc; knowing definition is enough}
   TCallBack3Ptr = ^TCallBack3;


   TagAry        = array [1..KnownTags] of PChar;

   LIPtr         = ^longint;
   LIAry         = array [0..cLIMax] of longint; {record selection array}
   LIAryPtr      = ^LIAry;

   NotaAry       = array[1..NotaMax] of PChar;
   NotaAryPtr    = ^NotaAry;

   MoveRec       = record
                      cMove    : string[6];
                      cTbl     : 0..4;
                      cOrd     : 5..250;
                   end;
   MoveAry       = array [1..MoveMax] of MoveRec;
   MoveAryPtr    = ^MoveAry;

   SfxRec        = record
                      cSfx     : string[5];  {tbl sorted inversly, c1, c2...c5 etc}
                      cOrd     : 5..250;
                   end;
   SfxAry        = array [1..SfxMax] of SfxRec;
   SfxAryPtr     = ^SfxAry;

   NameRec       = record
                      cName    : string[29]; 
                      cTbl     : 0..3;
                      cOrd     : 5..250;
                   end;
   NameAry       = array [1..NameMax] of NameRec;
   NameAryPtr    = ^NameAry;

   WordRec       = record
                      cWrd     : string[23]; 
                      cTbl     : 0..3;
                      cOrd     : 5..250;
                   end;
   WordAry       = array [1..WordMax] of WordRec;
   WordAryPtr    = ^WordAry;

   PCBufAry      = array [0..PCharStrLen] of char;
   PCBufPtr      = ^PCBufAry;
   MoveLocAry    = array [5..250] of integer; 

   {--Packing ONLY types (begin)--}
   ACode          = (other,bird,cat,dog);
   WordChopRec    = record
                       garb      : Str31; {non alpha, non-(  that precedes word}
                       ROStr     : Str63; {runout str; from start of NC word}
                       pG        : byte;  {encoded end of garbage; encapsulation}
                       pGLen     : byte;  {len by which garbage mite be reduced}
                       pCode     : ACode; {provisional BCD (bird/cat/dog) code}
                       rCode     : ACode; {revised BCD code}
                       Tbl       : byte;
                       TLen      : byte;
                       OB        : byte;    {OrdByte; in lieu of NDX}
                       trailB    : Boolean; {dog; following NC word has leading blank}
                       Used      : Boolean; {"this rec used" (for clearing) }
                    end;
   WChopAry       = array [1..MaxCWrds] of WordChopRec;
   WChopAryPtr    = ^WChopAry;
   Str86          = string[86]; 
   {--Packing ONLY types (end)----}

   RunInfoRec  = record
                    Abort666 : integer;
                    filler   : integer;
                    NA1      : NotaAryPtr;
                    NA2      : NotaAryPtr;
                    MA       : MoveAryPtr;
                    SA       : SfxAryPtr;
                    NA       : NameAryPtr;
                    NF       : NameAryPtr; {1st names only}  
                    WA       : WordAryPtr; {"words" in Site/Event}
                    WO       : WordAryPtr; {"words" in Opening}
                    PW       : WordAryPtr; {toggles between WA and WO}
                    origStr  : Str255; {allows "backout" to orig across procs}
                    TxfTo246 : Str255;
                    TxfFm246 : Str255;
                    TxfTo150 : Str255;
                    TxfFm150 : Str255;
                    TxfToLBA : Str255; {transform to low-bit Alpha (6 bits) }
                    TxfFmLBA : Str255; {transform fm low-bit Alpha (back to alphabetic) }
                    LogicErr : Str31;  {a place to put short FatalErrors}
                    SortDef  : Str31;
                    T0MLoc   : MoveLocAry; {MA}
                    T1MLoc   : MoveLocAry;
                    T2MLoc   : MoveLocAry;
                    T3MLoc   : MoveLocAry;
                    T4MLoc   : MoveLocAry;
                    T0SLoc   : MoveLocAry; {SA}
                    T0NLoc   : MoveLocAry; {NA}
                    T1NLoc   : MoveLocAry;
                    T2NLoc   : MoveLocAry;
                    T3NLoc   : MoveLocAry;
                    T0NFLoc  : MoveLocAry; {NF}
                    T1NFLoc  : MoveLocAry;
                    T2NFLoc  : MoveLocAry;
                    T3NFLoc  : MoveLocAry;
                    T0WALoc  : MoveLocAry; {WA}
                    T1WALoc  : MoveLocAry;
                    T2WALoc  : MoveLocAry;
                    T3WALoc  : MoveLocAry;
                    T0WOLoc  : MoveLocAry; {WO} 
                    T1WOLoc  : MoveLocAry;
                    T2WOLoc  : MoveLocAry;
                    T3WOLoc  : MoveLocAry;

                    WChop    : WChopAryPtr; {packing only; WA,WO,PW}
                    ECOStr   : PChar;
                    gBoardRI : pointer;     {position searching only}
                    gErrRI   : pointer;     {position searching only}
                    work1PC  : PChar; {if used, allocated to size PCharStrLen}
                                      {Use: Packing:   PackPGNFile}
                                      {     Unpacking: UnPackName }
                                      {                FileSort via RetrieveField(99)}
                    work2PC  : PChar; {ditto}
                                      {Use: Packing:   PackPGNFile  }
                                      {     Unpacking: UnPackPGNFile}
                                      {                UnPackPGNGame}
                                      {                FileSort     }
                                      {                QuerySelFile }
                    work3PC  : PChar; {ditto}
                                      {Use: Packing:   (none)       }
                                      {     Unpacking: UnPackPGNFile}
                                      {                FileSort     }
                                      {                FlUnPkLIAry  }
                    work4PC  : PChar; {ditto}
                                      {Use: Retr/Unpk: FlUnPkLIAry  }
                    FirstBadGame  : longint; {see FilePosSrchUnPack}
                    DisableChkErr : Boolean; {allows check/nocheck nota errors}
                    gfiller       : Str86;
                 end;
   RunInfoPtr  = ^RunInfoRec;

const

   cPGNTag : TagAry = (
    {1}  ('Event'),
    {2}  ('Site'),
    {3}  ('Date'),
    {4}  ('Round'),
    {5}  ('White'),
    {6}  ('Black'),
    {7}  ('Result'),
    {8}  ('ECO'),
    {9}  ('WhiteElo'),
   {10}  ('BlackElo'),
   {11}  ('Opening'),
   {12}  ('PlyCount'),
   {13}  ('FEN'),
   {14}  ('WhiteTitle'),
   {15}  ('BlackTitle'),
   {16}  ('WhiteCountry'),
   {17}  ('BlackCountry'),
   {18}  ('Annotator'));

var
   RI       : RunInfoPtr;

implementation

end.
